<?php

class ModuleDictionary extends AppModule
{
  function __construct(System $System)
  {
    parent::__construct($System);
    $this->Name = 'Dictionary';
    $this->Version = '1.0';
    $this->Creator = 'Chronos';
    $this->License = 'GNU/GPL';
    $this->Description = 'Custom dictionary for better translation coordination';
    $this->Dependencies = array('Translation');
  }

  function DoStart()
  {
    $this->System->RegisterPage('dictionary', 'PageDictionary');
    $this->System->RegisterMenuItem(array(
      'Name' => 'Dictionary',
      'Title' => T('Dictionary'),
      'Hint' => T('Dictionary words from WoW'),
      'Link' => $this->System->Link('/dictionary/'),
      'Permission' => LICENCE_ANONYMOUS,
      'Icon' => '',
    ), 1);
    if(array_key_exists('Search', $this->System->ModuleManager->Modules))
      $this->System->ModuleManager->Modules['Search']->RegisterSearch('dictionary',
      T('Dictionary'), array('Text', 'Description'),
     '(SELECT * FROM `Dictionary` WHERE `Language` = '.$this->System->Config['OriginalLanguage'].') AS `T`',
        $this->System->Link('/dictionary/?search='));
  }
}

class PageDictionary extends Page
{
  function WriteTranslatNames($Text, $mode)
  {
    $Output = '';
    if ($mode == 0) $Output .= '..'.T('Found names of items, places and NPC');
    if ($mode == 1) $Output .= '..'.T('All untranslated');
    if ($mode == 2) $Output .= '..'.T('Untranslated');
    $Output .= '<table class="BaseTable">';
    $Output .= '<tr><th>'.T('Original').'</th>'.'<th>'.T('Translation').'</th></tr>';

    // TODO: Remove static table names in code. They should be controlled from database
    $TablesColumn = array
    (
      'TextGameObject' => 'Name',
      'TextCreature' => 'name',
      'TextItem' => 'Name',
      'TextTransport' => 'Name',
      'TextAreaTriggerTeleport' => 'Name',
      'TextAreaTriggerTavern' => 'Name',
      'Dictionary' => 'Text',
      'TextArea' => 'Name',
      'TextAreaPOI' => 'Name',
      'TextCharacterClass' => 'Name',
      'TextCharacterRace' => 'Name1',
      'TextItemSubClass' => 'Name',
      'TextCreatureType' => 'Name',
    );

    $buff = GetTranslatNames($Text, $mode, GetTranslatNamesArray());

    // $buff[] = array($Line['ID'],GetIDbyName($Table),$Line[$Column],$Line['Tran']);
    foreach($buff as $Line)
    {
      if ($mode == 0)
      {
        if (strpos(strtolower($Text), strtolower($Line[2])) > 0)
        {
          $Output .= '<tr><td>'.$Line[2].'</td>';
          if ($Line[3] <> '') $Output .= '<td><a target="_NEW2" href="'.$this->System->Link('/form.php?group='.$Line[1].'&amp;ID='.$Line[0]).'">'.$Line[3].'</a></td></tr>';
          else $Output .= '<td><a target="_NEW2" href="'.$this->System->Link('/form.php?group='.$Line[1].'&amp;ID='.$Line[0]).'">'.T('Translate').'</a></td></tr>';
        }
      } else
      {
        $Output .= '<tr><td>'.$Line[2].'</td>';
        if ($Line[3] <> '') $Output .= '<td><a target="_NEW2" href="'.$this->System->Link('/form.php?group='.$Line[1].'&amp;ID='.$Line[0]).'">'.$Line[3].'</a></td></tr>';
          else $Output .= '<td><a target="_NEW2" href="'.$this->System->Link('/form.php?group='.$Line[1].'&amp;ID='.$Line[0]).'">'.T('Translate').'</a></td></tr>';
      }
    }
    $Output .= '</table>';
    return($Output);
  }

  function DictionaryInsert()
  {
    $Output = '';
    if($this->System->User->Licence(LICENCE_USER))
    {
      $Output .= '<form action="?action=save" method="post">'.
        '<fieldset><legend>'.T('New word insertion').'</legend>'.
        '<table><tr><td>'.
        T('Original word').':</td><td><input type="text" name="Original" /></td></tr>'.
        '<tr><td>'.T('Translated').':</td><td><input type="text" name="Translated" /></td></tr>'.
        '<tr><td>'.T('Language').':</td><td>'.WriteLanguages($_SESSION['language']).'</td></tr>'.
        '<tr><td>'.T('Selectable description').':</td><td><input type="text" name="Description" /></td></tr>'.
        '<tr><td colspan="2"><input type="submit" value="'.T('Save').'" /></td></tr>'.
        '</td></tr></table>'.
        '</fieldset>'.
        '</form>';
    } else $Output .= ShowMessage(T('Access denied'), MESSAGE_CRITICAL);
    return($Output);
  }

  function DictionarySave()
  {
    if($this->System->User->Licence(LICENCE_USER))
    {
      if(array_key_exists('Original', $_POST) and array_key_exists('Translated', $_POST) and array_key_exists('Description', $_POST))
      {
        // Check if original text exists and determine entry id
        $DbResult = $this->Database->query('SELECT * FROM `Dictionary` WHERE '.
          '`Text` = "'.$_POST['Original'].'" AND `Language`= '.$this->System->Config['OriginalLanguage']);
        if($DbResult->num_rows > 0)
        {
          $DbRow = $DbResult->fetch_assoc();
          $Entry = $DbRow['Entry'];
        } else
        {
          $DbResult = $this->Database->query('SELECT MAX(`Entry`) FROM `Dictionary`');
          $DbRow = $DbResult->fetch_row();
          $Entry = $DbRow[0] + 1;
          $this->Database->query('INSERT INTO `Dictionary` ( `Text` , `Entry` , `Description` , '.
            '`User`, `Language` ) VALUES ("'.$_POST['Original'].'", "'.$Entry.'", "", NULL, '.$this->System->Config['OriginalLanguage'].');');
        }

        $DbResult = $this->Database->query('SELECT `Id` FROM `Dictionary` WHERE '.
          '`Entry` = '.$Entry.' AND `Language`='.$_POST['Language'].' AND `User`='.$this->System->User->Id);
        if($DbResult->num_rows > 0)
        {
          $DbRow = $DbResult->fetch_assoc();
          $this->Database->query('UPDATE `Dictionary` SET `Text`="'.$_POST['Translated'].'", '.
            '`Description` = "'.$_POST['Description'].'" WHERE Id='.$DbRow['Id']);
        } else
          $this->Database->query('INSERT INTO `Dictionary` ( `Text` , `Entry` , `Description` , '.
            '`User`, `Language` ) VALUES ("'.$_POST['Translated'].'", "'.$Entry.'", "'.
            $_POST['Description'].'", '.$this->System->User->Id.', '.$_POST['Language'].')');
        $Output = ShowMessage('Záznam byl uložen!');
      } else $Output = ShowMessage(T('You have to fill all column of form.'), MESSAGE_CRITICAL);
    } else $Output = ShowMessage(T('Access denied'), MESSAGE_CRITICAL);
    return($Output);
  }

  function DictionaryRemove()
  {
    if($this->System->User->Licence(LICENCE_USER))
    {
      $this->Database->query('DELETE FROM `Dictionary` WHERE (`User`='.$this->System->User->Id.') AND (`Id`='.($_GET['id'] * 1).')');
      $Output = ShowMessage(T('Record removed'));
    } else $Output = ShowMessage(T('Access denied'), MESSAGE_CRITICAL);
    return($Output);
  }

  function DictionaryModify()
  {
    if($this->System->User->Licence(LICENCE_USER))
    {
      $DbResult = $this->Database->query('SELECT * FROM `Dictionary` WHERE `Id`='.($_GET['id'] * 1));
      if($DbResult->num_rows > 0)
      {
        $DbRow = $DbResult->fetch_assoc();
        $DbResult = $this->Database->query('SELECT * FROM `Dictionary` WHERE (`User` IS NULL) '.
          'AND (`Entry`='.$DbRow['Entry'].') AND (`Language`= '.$this->System->Config['OriginalLanguage'].')');
        $DbRow2 = $DbResult->fetch_assoc();
        $Output = '<form action="?action=save" method="post">'.
          '<fieldset><legend>Upravení slova</legend>'.
          '<input type="hidden" name="id"  value="'.$_GET['id'].'"/>'.
          '<table><tr><td>'.
          'Původní anglické slovo:</td><td><input type="text" name="Original" value="'.htmlspecialchars($DbRow2['Text']).'" /></td></tr>'.
          '<tr><td>Přeložené:</td><td><input type="text" name="Translated" value="'.htmlspecialchars($DbRow['Text']).'" /></td></tr>'.
          '<tr><td>'.T('Language').':</td><td>'.WriteLanguages($DbRow['Language']).'</td></tr>'.
          '<tr><td>'.T('Description').':</td><td><input type="text" name="Description"  value="'.htmlspecialchars($DbRow['Description']).'" /></td></tr>'.
          '<tr><td colspan="2"><input type="submit" value="'.T('Save').'" /></td></tr>'.
          '</td></tr></table>'.
          '</fieldset>'.
          '</form>';
      } else $Output = ShowMessage(T('Item not found'), MESSAGE_CRITICAL);
    } else $Output = ShowMessage(T('Access denied'), MESSAGE_CRITICAL);
    return($Output);
  }

  function DictionaryGroup()
  {
    $TranslationTree = $this->System->ModuleManager->Modules['Translation']->GetTranslationTree();

    $Output = '';
    $GroupId = LoadGroupIdParameter();
    if(array_key_exists('mode', $_GET)) $mode = $_GET['mode'];
    else $mode = 0; //0 = exact names, 1=all names, 2=nontranslated
    $Table = $TranslationTree[$GroupId]['TablePrefix'];

    if(array_key_exists('ID', $_GET))
    {
      $TextID = $_GET['ID'];
      $Output .= T('Translated names related to translation text').': <a target="_NEW" href="'.$this->System->Link('/form.php?group='.$GroupId.'&amp;ID='.$TextID).'">'.$TextID.'</a><br />'.
        '<a href="?action=group&amp;mode=1&amp;group='.$GroupId.'&amp;ID='.$TextID.'" title="'.T('Show all similar results').'">'.T('All').'</a> '.
        '<a href="?action=group&amp;mode=0&amp;group='.$GroupId.'&amp;ID='.$TextID.'" title="'.T('Show all exact results').'">'.T('Exact').'</a> '.
        '<a href="?action=group&amp;mode=2&amp;group='.$GroupId.'&amp;ID='.$TextID.'" title="'.T('Show all untranslated').'">'.T('Untranslated').'</a> ';
      $DbResult = $this->Database->query('SELECT * FROM '.$Table.' WHERE ID = '.$TextID);
      $Line = $DbResult->fetch_assoc();
      if(!$Line) $Output .= ShowMessage(T('Translation not found'), MESSAGE_CRITICAL);
      else
      {
        $Text = '';
        foreach($TranslationTree[$GroupId]['Items'] as $Index => $TextItem)
          $Text .= ' '.$Line[$TextItem['Column']];
        $Output .= $this->WriteTranslatNames($Text, $mode);
      }
    }
    return($Output);
  }

  function DictionaryShow()
  {
    global $LanguageList;

    $Output = '<form action="?" method="get" style="margin: 0px; padding: 0px;">'.
      '<table style="width: 100%; height: 100%;">';

    if(array_key_exists('search', $_GET)) $Search = $_GET['search'];
      else $Search = '';

    $Output .= '<tr><td>
      <input type="text" value="'.$Search.'" name="search" size="30" />
      <input type="submit" value="'.T('Search').'" />';
    if($this->System->User->Licence(LICENCE_USER))
      $Output .= ' <a href="?action=insert">'.T('Add word').'</a>';

    $Output .= '</td></tr>'.
      '<tr><td>'.T('Language').': ';
    $Lang = '<a href="?language=">'.T('All').'</a>';
    if($_SESSION['language'] == '') $Output .= '<strong>'.$Lang.'</strong> ';
      else $Output .= $Lang;
    foreach($LanguageList as $Language)
      if($Language['Enabled'] == 1)
      {
        $Lang = ' <a href="?language='.$Language['Id'].'">'.T($Language['Name']).'</a>';
        if($Language['Id'] == $_SESSION['language']) $Output .= '<strong>'.$Lang.'</strong> ';
          else $Output .= $Lang;
      }
    $Output .= '</td></tr>'.
      '<tr><td>';

    if(!is_numeric($_SESSION['language'])) $LanguageFilter = '';
      else $LanguageFilter = ' AND (`T1`.`Language` = '.$_SESSION['language'].')';

    if($Search <> '')
    {
      $Condition = ' AND (LOWER(`T1`.`Text`) LIKE LOWER("%'.$Search.'%")) OR '.
        ' (LOWER(`T2`.`Text`) LIKE LOWER("%'.$Search.'%")) OR '.
        '(LOWER(`T1`.`Description`) LIKE LOWER("%'.$Search.'%"))';
    } else $Condition = '';
    $sql = 'SELECT `User`.`Name` AS `UserName`, `User`.`Id` AS `UserId`, '.
      '`Language`.`Name` AS `LangName`, `T1`.`Id` AS `Id`, '.
      '`T1`.`Entry`, `T2`.`Text` AS `Original`, `T1`.`Text` AS `Translated`, `T1`.`Description` '.
      'FROM `Dictionary` AS `T1` JOIN `Dictionary` AS `T2` '.
      'ON (`T2`.`Entry` = `T1`.`Entry`) AND (`T2`.`Language` = '.$this->System->Config['OriginalLanguage'].') '.
      'JOIN `Language` ON `Language`.`Id` = `T1`.`Language` '.
      'JOIN `User` ON `User`.`ID` = `T1`.`User` '.
      'WHERE 1'.
      $LanguageFilter.$Condition;

    $DbResult = $this->Database->query('SELECT COUNT(*) FROM ('.$sql.') AS `T1`');
    $DbRow = $DbResult->fetch_row();
    $PageList = GetPageList($DbRow[0]);

    $Output .= $PageList['Output'];

    if(is_numeric($_SESSION['language'])) $LanguageName = $LanguageList[$_SESSION['language']]['Name'];
      else $LanguageName = T('Translation');
    $TableColumns = array(
      array('Name' => 'Original', 'Title' => T('English')),
      array('Name' => 'Translated', 'Title' => T($LanguageName)),
    );
    if(!is_numeric($_SESSION['language'])) $TableColumns[] = array('Name' => 'LangName', 'Title' => T('Language'));
    $TableColumns[] = array('Name' => 'Description', 'Title' => T('Description'));
    $TableColumns[] = array('Name' => 'UserName', 'Title' => T('Translator'));
    if($this->System->User->Licence(LICENCE_USER)) $TableColumns[] = array('Name' => '', 'Title' => T('Action'));
    $Order = GetOrderTableHeader($TableColumns, 'Original');
    $Output .= '<table class="BaseTable">'.$Order['Output'];

    $sql_page = $sql.$Order['SQL'].$PageList['SQLLimit'];
    $DbResult = $this->Database->query($sql_page);
    while($Line = $DbResult->fetch_assoc())
    {
      $Output .= '<tr>'.
        '<td>'.htmlspecialchars($Line['Original']).'</td>'.
        '<td><strong>'.htmlspecialchars($Line['Translated']).'</strong></td>';
      if(!is_numeric($_SESSION['language'])) $Output .= '<td>'.T($Line['LangName']).'</td>';
      $Output .= '<td>'.htmlspecialchars($Line['Description']).'</td>'.
        '<td><a href="'.$this->System->Link('/user/?user='.$Line['UserId']).'">'.
        $Line['UserName'].'</a></td>';
      if($this->System->User->Licence(LICENCE_USER))
      {
        if($Line['UserId'] == $this->System->User->Id)
          $Output .= '<td><a href="?action=remove&amp;id='.$Line['Id'].
            '" onclick="return confirmAction(\''.T('Do you really want to delete item?').'\');">'.T('Delete').'</a>'.
            ' <a href="?action=modify&amp;id='.$Line['Id'].'">'.T('Modify').'</a></td>';
          else $Output .= '<td></td>';
      }
      $Output .= '</tr>';
    }
    $Output .= '</table>'.
    $PageList['Output'].
      '</td></tr>'.
      '</table></form>';
    return($Output);
  }

  function Show()
  {
    global $LanguageList;

    $this->Title = T('Dictionary');

    $LanguageList = GetLanguageList();

    if(!isset($_SESSION['language']))
    {
      if($this->System->User->Licence(LICENCE_USER))
      {
        $_SESSION['language'] = $this->System->User->Language;
      } else $_SESSION['language'] = '';
    }
    if(array_key_exists('language', $_GET))
    {
      if($_GET['language'] == '') $_SESSION['language'] = '';
        else $_SESSION['language'] = $_GET['language'] * 1;
    }

    $Output = '<h3>'.T('Dictionary').'</h3>';

    $ShowList = true;
    if(array_key_exists('action', $_GET))
    {
      if($_GET['action'] == 'group')
      {
        $Output .= $this->DictionaryGroup();
        $ShowList = false;
      }
      else if($_GET['action'] == 'insert') $Output .= $this->DictionaryInsert();
      else if($_GET['action'] == 'save') $Output .= $this->DictionarySave();
      else if($_GET['action'] == 'remove') $Output .= $this->DictionaryRemove();
      else if($_GET['action'] == 'modify') $Output .= $this->DictionaryModify();
      else $Output .= ShowMessage(T('Unknown action'), MESSAGE_CRITICAL);
    }
    if($ShowList == true) $Output .= $this->DictionaryShow();
    return($Output);
  }
}